@@ -23,7 +23,14 @@ describe Tmuxinator::Cli do
       File.delete(local_project_path)
     end
   end
-  let(:cli) { Tmuxinator::Cli }
+
+  subject(:cli) { described_class }
+
+  let(:fixtures_dir) { File.expand_path("../../../fixtures/", __FILE__) }
+  let(:project) { FactoryBot.build(:project) }
+  let(:project_config) do
+    File.join(fixtures_dir, "sample_with_project_config.yml")
+  end
 
   before do
     ARGV.clear
@@ -201,6 +208,47 @@ describe Tmuxinator::Cli do
     end
   end
 
+  shared_examples_for :unsupported_version_message do |*args|
+    before do
+      ARGV.replace([*args])
+    end
+
+    context "unsupported version" do
+      before do
+        allow($stdin).to receive_messages(getc: "y")
+        allow(Tmuxinator::TmuxVersion).to receive(:supported?).and_return(false)
+      end
+
+      it "prints the warning" do
+        out, _err = capture_io { cli.start }
+        expect(out).to include "WARNING"
+      end
+
+      context "with --suppress-tmux-version-warning flag" do
+        before do
+          ARGV.replace([*args, "--suppress-tmux-version-warning"])
+        end
+
+        it "does not print the warning" do
+          out, _err = capture_io { cli.start }
+          expect(out).not_to include "WARNING"
+        end
+      end
+    end
+
+    context "supported version" do
+      before do
+        allow($stdin).to receive_messages(getc: "y")
+        allow(Tmuxinator::TmuxVersion).to receive(:supported?).and_return(true)
+      end
+
+      it "does not print the warning" do
+        out, _err = capture_io { cli.start }
+        expect(out).not_to include "WARNING"
+      end
+    end
+  end
+
   describe "#start" do
     before do
       ARGV.replace(["start", "foo"])
@@ -210,8 +258,6 @@ describe Tmuxinator::Cli do
     end
 
     context "no deprecations" do
-      let(:project) { FactoryBot.build(:project) }
-
       it "starts the project" do
         expect(Kernel).to receive(:exec)
         capture_io { cli.start }
@@ -251,6 +297,8 @@ describe Tmuxinator::Cli do
         expect(out).to include "DEPRECATION"
       end
     end
+
+    include_examples :unsupported_version_message, :start, :foo
   end
 
   describe "#stop" do
@@ -262,8 +310,6 @@ describe Tmuxinator::Cli do
     end
 
     context "with project name" do
-      let(:project) { FactoryBot.build(:project) }
-
       it "stop the project" do
         expect(Kernel).to receive(:exec)
         out, err = capture_io { cli.start }
@@ -271,18 +317,18 @@ describe Tmuxinator::Cli do
         expect(out).to eq ""
       end
     end
+
+    include_examples :unsupported_version_message, :stop, :foo
   end
 
   describe "#local" do
+    before do
+      allow(Tmuxinator::Config).to receive_messages(validate: project)
+      allow(Tmuxinator::Config).to receive_messages(version: 1.9)
+      allow(Kernel).to receive(:exec)
+    end
+
     shared_examples_for :local_project do
-      before do
-        allow(Tmuxinator::Config).to receive_messages(validate: project)
-        allow(Tmuxinator::Config).to receive_messages(version: 1.9)
-        allow(Kernel).to receive(:exec)
-      end
-
-      let(:project) { FactoryBot.build(:project) }
-
       it "starts the project" do
         expect(Kernel).to receive(:exec)
         out, err = capture_io { cli.start }
@@ -304,6 +350,8 @@ describe Tmuxinator::Cli do
       end
       it_should_behave_like :local_project
     end
+
+    include_examples :unsupported_version_message, :local
   end
 
   describe "#start(custom_name)" do
@@ -315,8 +363,6 @@ describe Tmuxinator::Cli do
     end
 
     context "no deprecations" do
-      let(:project) { FactoryBot.build(:project) }
-
       it "starts the project" do
         expect(Kernel).to receive(:exec)
         capture_io { cli.start }
@@ -324,24 +370,39 @@ describe Tmuxinator::Cli do
     end
   end
 
-  describe "#start(with project config flag)" do
+  describe "#start(with project config file)" do
     before do
+      allow(Tmuxinator::Config).to receive(:validate).and_call_original
       allow(Tmuxinator::Config).to receive_messages(version: 1.9)
+      allow(Kernel).to receive(:exec)
     end
 
-    let(:fixtures_dir) { File.expand_path("../../../fixtures/", __FILE__) }
-    let(:project_config) { File.join(fixtures_dir, "sample.yml") }
-
     context "no deprecations" do
-      it "doesn't start the project if given a bogus project config" do
+      it "starts the project if given a valid project config file" do
+        ARGV.replace(["start", "--project-config=#{project_config}"])
+        expect(Kernel).to receive(:exec)
+        capture_io { cli.start }
+      end
+
+      it "does not start the project if given a bogus project config file" do
         ARGV.replace(["start", "--project-config=bogus.yml"])
         expect(Kernel).not_to receive(:exec)
         expect { capture_io { cli.start } }.to raise_error(SystemExit)
       end
 
-      it "starts the project if given a project config" do
+      it "passes additional arguments through" do
+        ARGV.replace(["start", "--project-config=#{project_config}", "extra"])
+        expect(Tmuxinator::Config).
+          to(receive(:validate).
+             with(hash_including(args: array_including("extra"))))
+        capture_io { cli.start }
+      end
+
+      it "does not set the project name" do
         ARGV.replace(["start", "--project-config=#{project_config}"])
-        expect(Kernel).to receive(:exec)
+        expect(Tmuxinator::Config).
+          to(receive(:validate).
+             with(hash_including(name: nil)))
         capture_io { cli.start }
       end
     end
@@ -361,7 +422,7 @@ describe Tmuxinator::Cli do
         expect(File).not_to exist(path)
 
         # now generate a project file
-        expect(Tmuxinator::Cli.new.generate_project_file(name, path)).to eq path
+        expect(described_class.new.generate_project_file(name, path)).to eq path
         expect(File).to exist path
 
         # add some content to the project file
@@ -538,42 +599,60 @@ describe Tmuxinator::Cli do
   end
 
   describe "#debug" do
-    let(:project) { FactoryBot.build(:project) }
-    let(:project_with_force_attach) do
-      FactoryBot.build(:project_with_force_attach)
-    end
-    let(:project_with_force_detach) do
-      FactoryBot.build(:project_with_force_detach)
+    context "named project" do
+      let(:project_with_force_attach) do
+        FactoryBot.build(:project_with_force_attach)
+      end
+      let(:project_with_force_detach) do
+        FactoryBot.build(:project_with_force_detach)
+      end
+
+      before do
+        allow(Tmuxinator::Config).to receive_messages(validate: project)
+        expect(project).to receive(:render)
+      end
+
+      it "renders the project" do
+        ARGV.replace(["debug", "foo"])
+        capture_io { cli.start }
+      end
+
+      it "force attach renders the project with attach code" do
+        ARGV.replace(["debug", "--attach=true", "sample"])
+        capture_io { cli.start }
+        # Currently no project is rendered at all,
+        #   because the project file is not found
+        # expect(out).to include "attach-session"
+      end
+
+      it "force detach renders the project without attach code" do
+        ARGV.replace(["debug", "--attach=false", "sample"])
+        capture_io { cli.start }
+        # Currently no project is rendered at all
+        # expect(out).to_not include "attach-session"
+      end
+
+      it "renders the project with custom session" do
+        ARGV.replace(["debug", "sample", "bar"])
+        capture_io { cli.start }
+      end
     end
 
-    before do
-      allow(Tmuxinator::Config).to receive_messages(validate: project)
-      expect(project).to receive(:render)
-    end
+    context "project config file" do
+      before do
+        allow(Tmuxinator::Config).to receive_messages(version: 1.9)
+        expect(Tmuxinator::Config).to receive(:validate).and_call_original
+      end
 
-    it "renders the project" do
-      ARGV.replace(["debug", "foo"])
-      capture_io { cli.start }
-    end
+      it "renders the project if given a valid project config file" do
+        ARGV.replace(["debug", "--project-config=#{project_config}"])
+        expect { cli.start }.to output(/sample_with_project_config/).to_stdout
+      end
 
-    it "force attach renders the project with attach code" do
-      ARGV.replace(["debug", "--attach=true", "sample"])
-      capture_io { cli.start }
-      # Currently no project is rendered at all,
-      #   because the project file is not found
-      # expect(out).to include "attach-session"
-    end
-
-    it "force detach renders the project without attach code" do
-      ARGV.replace(["debug", "--attach=false", "sample"])
-      capture_io { cli.start }
-      # Currently no project is rendered at all
-      # expect(out).to_not include "attach-session"
-    end
-
-    it "renders the project with custom session" do
-      ARGV.replace(["debug", "sample", "bar"])
-      capture_io { cli.start }
+      it "does not render the project if given a bogus project config file" do
+        ARGV.replace(["start", "--project-config=bogus.yml"])
+        expect { capture_io { cli.start } }.to raise_error(SystemExit)
+      end
     end
   end
 
@@ -704,12 +783,23 @@ describe Tmuxinator::Cli do
 
   describe "#list" do
     before do
-      ARGV.replace(["list"])
       allow(Dir).to receive_messages(:[] => ["/path/to/project.yml"])
     end
 
-    it "lists all projects" do
-      expect { capture_io { cli.start } }.to_not raise_error
+    context "set --newline flag " do
+      ARGV.replace(["list --newline"])
+
+      it "force output to be one entry per line" do
+        expect { capture_io { cli.start } }.to_not raise_error
+      end
+    end
+
+    context "no arguments are given" do
+      ARGV.replace(["list"])
+
+      it "lists all projects " do
+        expect { capture_io { cli.start } }.to_not raise_error
+      end
     end
   end
 
@@ -751,7 +841,7 @@ describe Tmuxinator::Cli do
       end
 
       it "should generate a project file" do
-        new_path = Tmuxinator::Cli.new.find_project_file(name, false)
+        new_path = described_class.new.find_project_file(name, false)
         expect(new_path).to eq path
         expect(File).to exist new_path
       end
@@ -762,7 +852,7 @@ describe Tmuxinator::Cli do
 
       before do
         expect(File).not_to exist(path), "expected file at #{path} not to exist"
-        expect(Tmuxinator::Cli.new.generate_project_file(name, path)).to eq path
+        expect(described_class.new.generate_project_file(name, path)).to eq path
         expect(File).to exist path
 
         File.open(path, "w") do |f|
@@ -773,7 +863,7 @@ describe Tmuxinator::Cli do
       end
 
       it "should _not_ generate a new project file" do
-        new_path = Tmuxinator::Cli.new.find_project_file(name, false)
+        new_path = described_class.new.find_project_file(name, false)
         expect(new_path).to eq path
         expect(File).to exist new_path
         expect(File.read(new_path)).to match %r{#{extra}}
@@ -788,7 +878,7 @@ describe Tmuxinator::Cli do
       Dir.mktmpdir do |dir|
         path = "#{dir}/#{name}.yml"
         expect(File).not_to exist(path), "expected file at #{path} not to exist"
-        new_path = Tmuxinator::Cli.new.generate_project_file(name, path)
+        new_path = described_class.new.generate_project_file(name, path)
         expect(new_path).to eq path
         expect(File).to exist new_path
       end
@@ -799,7 +889,7 @@ describe Tmuxinator::Cli do
       allow(File).to receive(:open) { |&block| block.yield file }
       Dir.mktmpdir do |dir|
         path = "#{dir}/#{name}.yml"
-        _ = Tmuxinator::Cli.new.generate_project_file(name, path)
+        _ = described_class.new.generate_project_file(name, path)
         expect(file.string).to match %r{\A# #{path}$}
       end
     end
@@ -837,18 +927,18 @@ describe Tmuxinator::Cli do
     context "attach option" do
       describe "detach" do
         it "sets force_detach to false when no attach argument is provided" do
-          project = Tmuxinator::Cli.new.create_project(name: name)
+          project = described_class.new.create_project(name: name)
           expect(project.force_detach).to eq(false)
         end
 
         it "sets force_detach to true when 'attach: false' is provided" do
-          project = Tmuxinator::Cli.new.create_project(attach: false,
+          project = described_class.new.create_project(attach: false,
                                                        name: name)
           expect(project.force_detach).to eq(true)
         end
 
         it "sets force_detach to false when 'attach: true' is provided" do
-          project = Tmuxinator::Cli.new.create_project(attach: true,
+          project = described_class.new.create_project(attach: true,
                                                        name: name)
           expect(project.force_detach).to eq(false)
         end
@@ -856,18 +946,18 @@ describe Tmuxinator::Cli do
 
       describe "attach" do
         it "sets force_attach to false when no attach argument is provided" do
-          project = Tmuxinator::Cli.new.create_project(name: name)
+          project = described_class.new.create_project(name: name)
           expect(project.force_attach).to eq(false)
         end
 
         it "sets force_attach to true when 'attach: true' is provided" do
-          project = Tmuxinator::Cli.new.create_project(attach: true,
+          project = described_class.new.create_project(attach: true,
                                                        name: name)
           expect(project.force_attach).to eq(true)
         end
 
         it "sets force_attach to false when 'attach: false' is provided" do
-          project = Tmuxinator::Cli.new.create_project(attach: false,
+          project = described_class.new.create_project(attach: false,
                                                        name: name)
           expect(project.force_attach).to eq(false)
         end
